//------------------------------------------------------
// Functions for encoders reading
// Pins initialization
// Timer initialization
// ISR : for encoders reading in background
// readEncodeurs : to read ecoders values if any changes
//------------------------------------------------------

//-------------------------------------------
// read switche encoder value
// return 0 if nothing happeneed
// Ne permet qu'une saisie quand un bouton
// est appuyé
//-------------------------------------------
byte readButtons() {
int c = 0;
	if (!bitRead(PIND,PIN_ENCODER_SW)) {
		if (lastBoutonNav) {			// if the button was already pressed
			lastBoutonNav=B_MAINT;		// then this is a long press
			return B_MAINT;
		}
		// else we have to debounce
		// we wait for 10 ms for this
		delay(10);
		if (!bitRead(PIND,PIN_ENCODER_SW)) {	// if the bit is still 1
			lastBoutonNav=B_ENC1;			// we are sure that the button is 'really' pressed
			return B_ENC1;
		}
	}
	lastBoutonNav=0;
	return 0;
}

//--------------------------
// Encoders initialization
// Pins
// ISR each 2ms
//--------------------------
void init_encoders(void) {
	// Pin initialization for encoders
	//------------------------------------------
	for(int i=0;i<NB_ENCODEURS;i++) {
		pinMode(pinA[i], INPUT);		// define pin A inputs
		pinMode(pinB[i], INPUT);		// define pin B inputs
		digitalWrite(pinA[i],HIGH);		// Pull-up ON for pins A
		digitalWrite(pinB[i],HIGH);		// Pull-up ON for pins B
	}
	pinMode(PIN_ENCODER_SW, INPUT);		// define encoder switch pin
	digitalWrite(PIN_ENCODER_SW,HIGH);	// Pull-up ON for switch pin
	TIMSK1 &= ~(1<<TOIE1);				// Timer/Counter1, Overflow Interrupt Disable
	TCCR1A &= ~((1<<WGM11) | (1<<WGM10));	// timer/counter mode normal
	TCCR1B &= ~(1<<WGM12);  				//
	TIMSK1 &= ~(1<<OCIE1A);					// Output Compare A Match Interrupt Disable  
	// Prescaler a 64
	TCCR1B |= (1<<CS10)  | (1<<CS11) ; // CLKio/64 (from prescaler)
	TCCR1B &= ~(1<<CS12);
	tcnt1 = 65037;						// init temp counter for 2ms
	TCNT1 = tcnt1;						// init real counter timer 1
	TIMSK1 |= (1<<TOIE1);				// Timer/Counter1, Overflow Interrupt Enable
}

//----------------------------------------------------- 
// HANDLER LECTURE ENCODEURS
// Procedure geree par timer 1
//----------------------------------------------------- 
ISR(TIMER1_OVF_vect) {  
int MSB;
int LSB;
int encoded;
int sum;
byte i;
	TCNT1 = tcnt1;      // Reload du timer
		
	for (i=0;i<NB_ENCODEURS;i++) {  
		//MSB = digitalRead(pinA[i]); // MSB = most significant bit
		//LSB = digitalRead(pinB[i]); // LSB = least significant bit
		MSB = bitRead(PIND,pinA[i]);	// MSB = most significant bit
		LSB = bitRead(PIND,pinB[i]);	// LSB = least significant bit
		encoded = (MSB << 1) |LSB;		// conversion
		// A 4 bits sum variable is build
		// the 2 most significant bits are the last read
		// the 2 last significant bits are the current read
		sum  = (lastEncoded[i] << 2) | encoded; // Build 4 bits sum
		// Test if we turn CW
		if(sum == 0b1101 || sum == 0b0100 || sum == 0b0010 || sum == 0b1011)
			counter[i]++;
		// Test if we turn CCW 
		if(sum == 0b1110 || sum == 0b0111 || sum == 0b0001 || sum == 0b1000)
			counter[i]--;
		lastEncoded[i] = encoded;   // Store current state for next time
		// Depend on encoder type used
		// sometime you need to divide by 2 and sometime divide by 4
		// bcounter[i]=counter[i]>>2;  // Counter divided by 4
		bcounter[i]=counter[i]>>1;  // Counter divided by 2
	}		

}

//--------------------------------------------
// Routine principale de lecture des encodeurs
//--------------------------------------------
byte readEncodeurs(void) {
byte i,val;
  val=0;
  // Lire les 2 encodeurs
  for (i=0;i<NB_ENCODEURS;i++) {
    // si la valeur a variee pour un des encodeurs
    if (lcounter[i]!=bcounter[i]) {
      // Tester le send
      // Si UP valeur de 11 a 14
      if (bcounter[i]>lcounter[i])
        val=i+11;
      // Si down valeur de 1 a 4
      else
        val=i+1;
      lcounter[i]=bcounter[i];
      return val;
    }
  }
  return 0;
}

